<?php

namespace Remytek\Docbuilder\Policies;

use Litepie\User\Contracts\UserPolicy;
use Remytek\Docbuilder\Models\Docbuilder;

class DocbuilderPolicy
{

    /**
     * Determine if the given user can view the docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function view(UserPolicy $user, Docbuilder $docbuilder)
    {
        if ($user->canDo('docbuilder.docbuilder.view') && $user->isAdmin()) {
            return true;
        }

        return $docbuilder->user_id == user_id() && $docbuilder->user_type == user_type();
    }

    /**
     * Determine if the given user can create a docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('docbuilder.docbuilder.create');
    }

    /**
     * Determine if the given user can update the given docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function update(UserPolicy $user, Docbuilder $docbuilder)
    {
        if ($user->canDo('docbuilder.docbuilder.edit') && $user->isAdmin()) {
            return true;
        }

        return $docbuilder->user_id == user_id() && $docbuilder->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Docbuilder $docbuilder)
    {
        return $docbuilder->user_id == user_id() && $docbuilder->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Docbuilder $docbuilder)
    {
        if ($user->canDo('docbuilder.docbuilder.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given docbuilder.
     *
     * @param UserPolicy $user
     * @param Docbuilder $docbuilder
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Docbuilder $docbuilder)
    {
        if ($user->canDo('docbuilder.docbuilder.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
