<?php

namespace Remytek\Docbuilder\Providers;

use Illuminate\Support\ServiceProvider;

class DocbuilderServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'docbuilder');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'docbuilder');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerDocbuilder();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerDocbuilder()
    {
        $this->app->bind('docbuilder', function($app) {
            return new Docbuilder($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Docbuilder', 'Lavalite\Docbuilder\Facades\Docbuilder');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('remytek.docbuilder', function ($app) {
            return $this->app->make('Remytek\Docbuilder\Docbuilder');
        });

                // Bind Docbuilder to repository
        $this->app->bind(
            'Remytek\Docbuilder\Interfaces\DocbuilderRepositoryInterface',
            \Remytek\Docbuilder\Repositories\Eloquent\DocbuilderRepository::class
        );

        $this->app->register(\Remytek\Docbuilder\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Remytek\Docbuilder\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and docbuilder's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/docbuilder.php', 'docbuilder'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeDocbuilder::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['remytek.docbuilder'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('remytek/docbuilder.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/docbuilder')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/docbuilder')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
